<?php
/**
 * Player Class Definition Page
 *
 * This file contains the definition for the Player class.
 * @package The_Theologian_Game
 * @author Andrew D. Goetz
 * @copyright  Andrew D. Goetz
 * @license http://creativecommons.org/licenses/by-sa/2.5/legalcode
 */

/**
 * Player Class Definition
 *
 * This class defines the player object.  It contains the player's  name, 
 * game image, score, and completed quests.
 * @package The_Theologian_Game
 * @author Andrew D. Goetz
 */
class Player {
	/************************************************************************
								Properties
	************************************************************************/
	/**
	 * The Player's Name, which they give
	 * @var string
	 */
	protected $name;

	/**
	 * The path to the Player's Avatar
	 * @var string
	 */
	protected $avatar;

	/**
	 * The player's current score
	 * @var int
	 */
	protected $score;

	/**
	 * The player's aquired "metals" showing what they have accomplished.  It is
	 * an array with the given form:
	 *	$metals['angelology'] bool
	 *	$metals['anthropology'] bool
	 *	$metals['hamartiology'] bool
	 *	$metals['inquisition'] bool
	 * @var array
	 */
	public $metals;

	/************************************************************************
								Methods
	************************************************************************/
	/**
	 * Create an instance of a Player
	 * @return void
	 */
	public function __construct() {
		// Give a default Name
		$this->name = 'The Theologian';
		// Give a default Avatar
		$this->avatar = 'images/male.jpg';
		// Make sure that the score is set to 0
		$this->score = 0;
		// Make sure that the metals is a propery formatted array
		$this->metals = array('angelology' => false,
							  'anthropology' => false,
							  'hamartiology' => false,
							  'inquisition' => false);
	} // end fuction __construct

	/************************************************************************
							Setter and Getters
	************************************************************************/
	/**
	 * Set a value into the name property
	 * @param string $string
	 * @return bool
	 */
	public function setName($string) {
		// Make sure that it's a string
		$string = (string) $string;
		$this->name = $string;
		return true;
	} // end function setName()
	
	/**
	 * Get the name 
	 * @return string
	 */
	public function getName() {
		return $this->name;
	} // end function getName()
	
	/**
	 * Set a value into the avatar property
	 * @param string $string
	 * @return bool
	 */
	public function setAvatar($string) {
		// Make sure that it's a string
		$string = (string) $string;
		$this->avatar = $string;
		return true;
	} // end function setAvatar()
	
	/**
	 * Get the avatar 
	 * @return string
	 */
	public function getAvatar() {
		return $this->avatar;
	} // end function getAvatar()
	
	/**
	 * Set a value into the score parameter
	 * @param int $int
	 * @return bool
	 */
	public function setScore($int) {
		// Make sure that it's a valid score
		$int = (int) $int;
		$this->score = $int;
		return true;
	} // end function setScore()
	
	/**
	 * Get the Score 
	 * @return int
	 */
	public function getScore() {
		return $this->score;
	} // end function getScore()
	
	/************************************************************************
							Interaction Methods 
	************************************************************************/
	/**
	 * Complete a Quest
	 *
	 * Once a user has completed a quest, that type of quest can be passed to this
	 * method, and it will set that metal to true.
	 * @param string $string
	 * @return bool
	 */
	public function completeQuest($string) {
		// Check to see if the string is valid
		if (!in_array($string, array('angelology', 'anthropology', 'hamartiology', 'inquisition'))) return false;
		
		// Set the correct type to true
		$this->metals[$string] = true;
		return true;
	} // end function completeQuest()

	/**
	 * Add a value to the score
	 * 
	 * This method will take either positive or negative integers which will be
	 * added to the score to create a new score, which will be set into the
	 * players score
	 * @param int $int
	 * @return bool
	 */
	public function addToScore($int) {
		// Make sure that the given score addition is an integer
		$int = (int) $int;
		return $this->setScore($this->getScore() + $int);
	} // end function addToScore()

	/************************************************************************
							Output Methods 
	************************************************************************/
	/**
	 * Output this Player's scores
	 *
	 * This method will output a div which gives the player's current score
	 * and shows which quests he has completed.
	 * @return string
	 */
	public function toScore() {
		$string = "<div>
	Score: " . $this->getScore() . "<br />
	Completed Quests: ";
		// Add each of the completed quests
		$completed = array();
		foreach ($this->metals as $key=>$metal) {
			if ($metal) $completed[] = $key;
		} // end addition of each metal
		$string .= join(', ', $completed);
		if (count($completed) == 0) $string .= 'none';
		$string .= "</div>\r\n";
		return $string;
	} // end function toScore()
	
} // end creation of Player class