<?php
/**
 * Question Class Definition Page
 *
 * This file contains the definition for the Question class.
 * @package The_Theologian_Game
 * @author Andrew D. Goetz
 * @copyright  Andrew D. Goetz
 * @license http://creativecommons.org/licenses/by-sa/2.5/legalcode
 */

/**
 * Question Class Definition
 *
 * This class comtains all of the data related to a single question.  It can
 * query the database for all of it's information, including it's answers.  It
 * also can keep track of whether the question was answered correctly or not.
 * @package The_Theologian_Game
 * @author Andrew D. Goetz
 */
class Question {
	/************************************************************************
								Properties
	************************************************************************/
	/**
	 * Question ID from the database
	 * @var int
	 */
	protected $id;

	/**
	 * The text of the question to be asked
	 * @var string
	 */
	protected $question;

	/**
	 * The type of question this is.  The valid types are: angelology, anthropology
	 * and hamartiology.
	 * @var string
	 */
	protected $class;

	/**
	 * An integer representing the difficulty for the question.  This is an int
	 * between 0 and 2
	 * @var int
	 */
	protected $difficulty;

	/**
	 * The relative path to the image for this question.
	 * @var string
	 */
	protected $image;

	/**
	 * If this question is marked as correct or not
	 * @var bool
	 */
	protected $is_correct;

	/**
	 * An array of the answers for this question.  It will have this structure:
	 *	$answers[id][answer]
	 *	$answers[id][is_correct]
	 *	$answers[id][order]
	 * @var array
	 */
	protected $answers;
	
	/************************************************************************
								Methods
	************************************************************************/
	/**
	 * Create an instance of this Question
	 * @param int $id
	 * @return void
	 */
	public function __construct($id = 0) {
		// Make sure that the answers is an array
		$this->answers = array();
		// If the ID is greater than 0, load it from the database
		$id = (int) $id;
		if ($id > 0) $this->loadFromDb($id);
	} // end fuction __construct
	
	/************************************************************************
							Setter and Getters
	************************************************************************/
	/**
	 * Set a value into the id parameter
	 *
	 * This method is private because we will use the ID setting as a test to
	 * see if this has been loaded from the database.
	 * @param int $int
	 * @return bool
	 */
	protected function setId($int) {
		// Make sure that it's a valid ID
		$int = (int) $int;
		if ($int > 0) {
			$this->id = $int;
			return true;
		} // end actions if the ID is valid
		return false;
	} // end function setHeader()
	
	/**
	 * Get the ID 
	 * @return int
	 */
	public function getId() {
		return $this->id;
	} // end function getHeader()
	
	/**
	 * Set a value into the question property
	 * @param string $string
	 * @return bool
	 */
	public function setQuestion($string) {
		// Make sure that it's a string
		$string = (string) $string;
		$this->question = $string;
		return true;
	} // end function setQuestion()
	
	/**
	 * Get the question 
	 * @return string
	 */
	public function getQuestion() {
		return $this->question;
	} // end function getQuestion()
	
	/**
	 * Set a value into the class property
	 *
	 * This will make sure that the passed value is either angelology, anthropology,
	 * or hamartiology.
	 * @param string $string
	 * @return bool
	 */
	public function setClass($string) {
		// Make sure that it's a valid result
		$string = (string) $string;
		if (in_array($string, array('angelology', 'anthropology', 'hamartiology', 'inquisition'))) {
			$this->class = $string;
			return true;
		} // end setting, if valid
		return false;
	} // end function setClass()
	
	/**
	 * Get the class 
	 * @return string
	 */
	public function getClass() {
		return $this->class;
	} // end function getClass()
	
	/**
	 * Set a value into the difficulty parameter
	 *
	 * This will check to see if the int given is between 0 and 2.
	 * @param int $int
	 * @return bool
	 */
	public function setDifficulty($int) {
		// Make sure that it's a valid difficulty
		$int = (int) $int;
		if ($int >= 0 && $int <= 2) {
			$this->difficulty = $int;
			return true;
		} // end actions if the int is valid
		return false;
	} // end function setDifficulty()
	
	/**
	 * Get the Difficulty 
	 * @return int
	 */
	public function getDifficulty() {
		return $this->difficulty;
	} // end function getDifficulty()
	
	/**
	 * Set a value into the image property
	 * @param string $string
	 * @return bool
	 */
	public function setImage($string) {
		// Make sure that it's a string
		$string = (string) $string;
		$this->image = $string;
		return true;
	} // end function setImage()
	
	/**
	 * Get the image 
	 * @return string
	 */
	public function getImage() {
		return $this->image;
	} // end function getImage()
	
	/**
	 * Set a value into the is_correct property
	 * @param bool $bool
	 * @return bool
	 */
	public function setIsCorrect($bool) {
		// Make sure that it's a bool
		$bool = (bool) $bool;
		$this->is_correct = $bool;
		return true;
	} // end function setIsCorrect()
	
	/**
	 * Get the is_correct 
	 * @return bool
	 */
	public function getIsCorrect() {
		return $this->is_correct;
	} // end function getIsCorrect()
	
	/************************************************************************
							Array Accessors 
	************************************************************************/
	/**
	 * Add an answer to the answers array
	 * 
	 * This method will check that the data passed to it will create a valid
	 * answer, and then, if so, add it to the list of answers for this question.
	 * @param int $id
	 * @param string $answer
	 * @param bool $is_correct
	 * @param int $order
	 * @return bool
	 */
	public function addAnswer($id, $answer, $is_correct = false, $order = 0) {
		// The only way to fail this is to give an invalid ID
		$id = (int) $id;
		if ($id <= 0) return false;
		
		// Now, make sure that the rest of the data is valid
		$answer = (string) $answer;
		$is_correct = (bool) $is_correct;
		$order = (int) $order;
		
		// Add this answer to the list
		$this->answers[$id] = array('answer' => $answer,
									'is_correct' => $is_correct,
									'order' => $order);
		return true;
	} // end function addAnswer()
	
	/**
	 * Remove an answer
	 *
	 * If the given ID of an answer is given, it will remove that answer from
	 * from the current list
	 * @param int $id
	 * @return bool
	 */
	public function removeAnswer($id) {
		// Make sure that the ID is an int
		$id = (int) $id;
		// If the item is there, remove it
		if (isset($this->answers[$id])) {
			unset($this->answers[$id]);
			return true;
		} // end removal of the answer
		return false;
	} // end function removeAnswer()
	
	/**
	 * Compare two answers and see which comes first
	 * @param array $first
	 * @param array $second
	 * @return int
	 */
	private function compareAnswers($first, $second) {
		return strcmp($first['order'], $second['order']);
	} // end function compareAnswers()
	
	/**
	 * Reorder the Answers so that they are in the right order
	 * @return void
	 */
	public function orderAnswers() {
		return uasort($this->answers, array('Question', 'compareAnswers'));
	} // end function orderAnswers
	
	/**
	 * Get all of the answers for this question
	 * @return array
	 */
	public function getAnswers() {
		return $this->answers;
	} // end function getAnswers()

	/************************************************************************
							Database Interactions 
	************************************************************************/
	/**
	 * Load a given ID from the database
	 * 
	 * Given a valid ID, this method will query the database, get all of the
	 * data for that question, and then set it into the object
	 * @param int $id
	 * @return bool
	 */
	public function loadFromDb($id) {
		// Make sure that the given ID is valid
		$id = (int) $id;
		if ($id <= 0) return false;

		// Create the SQL to get the data from the database
		$sql = "SELECT id,
					   question,
					   class,
					   difficulty,
					   image
				FROM questions
				WHERE id = '" . pg_escape_string($id) . "'";

		// Query the database
		$select = pg_query($GLOBALS['_SCRIPT']['database'], $sql);
		// If it didn't work give up
		if (!$select || pg_num_rows($select) != 1) return false;
		
		// Walk through the result and set the parameters
		$row = pg_fetch_array($select, 0, PGSQL_ASSOC);
		$check['id'] = $this->setId($row['id']);
		$check['question'] = $this->setQuestion($row['question']);
		$check['class'] = $this->setClass($row['class']);
		$check['difficulty'] = $this->setDifficulty($row['difficulty']);
		$check['image'] = $this->setImage($row['image']);
		
		// If any of the settings didn't work, return false
		if (in_array(false, $check)) return false;
		
		// Now create the sql for the answers
		$sql = "SELECT id, 
					   answer,
					   is_correct,
					   answer_order
				FROM answers
				WHERE question = '" . pg_escape_string($id) . "'";
		// Query the database
		$select = pg_query($GLOBALS['_SCRIPT']['database'], $sql);
		// If it didn't work, give up
		if (!$select) return false;
		
		// Walk thorugh the results and add each to the answers
		$success = true;
		$count = pg_num_rows($select);
		for ($iterate = 0; $iterate < $count; $iterate++) {
			$row = pg_fetch_array($select, $iterate, PGSQL_ASSOC);
			// Set the correct booleans
			if ($row['is_correct'] == 't') $row['is_correct'] = true;
			else $row['is_correct'] = false;
			$test = $this->addAnswer($row['id'], $row['answer'], $row['is_correct'], $row['answer_order']);
			if (!$test) $success = false;
		} // end walking through the results of the answer query
		// Order the answers correctly
		$this->orderAnswers();
		return $success;
	} // end function loadFromDb()
	
	/************************************************************************
							Interaction Methods 
	************************************************************************/
	/**
	 * Given an answer ID, check and see if it is correct.  Mark this question
	 * accordingly.
	 * @param int $answer
	 * @return bool
	 */
	public function answerQuestion($int) {
		// Make sure that the int given is a valid answer int
		$int = (int) $int;
		if ($int <= 0 || !isset($this->answers[$int])) return false;
		
		// Find the value of the answer, set it for the question, and return the answer
		$answer = $this->answers[$int]['is_correct'];
		$this->setIsCorrect($answer);
		return $answer;
	} // end function answerQuestion()
	
} // end creation of class Question