<?php
/**
 * QuestionList Class Definition Page
 *
 * This file contains the definition for the QuestionList class.
 * @package The_Theologian_Game
 * @author Andrew D. Goetz
 * @copyright  Andrew D. Goetz
 * @license http://creativecommons.org/licenses/by-sa/2.5/legalcode
 */

/**
 * QuestionList Class Definition
 *
 * This class contains a smart list of Question objects.  It can be foreached
 * just as an array, but it also can return basic information about the objects
 * it contains such as the percentage answered or the number incorrect.
 * @package The_Theologian_Game
 * @author Andrew D. Goetz
 */
class QuestionList implements IteratorAggregate{
	/************************************************************************
								Properties
	************************************************************************/
	/**
	 * List type.  It can be angelology, anthropology, hamartiology, or inquisition
	 * @var string
	 */
	protected $type;
	
	/**
	 * List of Questions
	 * @var array
	 */
	protected $questions;
	
	/************************************************************************
								Methods
	************************************************************************/
	/**
	 * Create an instance of this Question List
	 * @param string $type
	 * @return void
	 */
	public function __construct($type = '') {
		// Make sure that the questions is an array
		$this->questions = array();
		// If a type is given, load that type's questions
		$type = (string) $type;
		if (in_array($type, array('angelology', 'anthropology', 'hamartiology', 'inquisition'))) $this->loadType($type);
	} // end fuction __construct
	
	/**
	 * Get an Iterator
	 *
	 * This method returns an iterator so that we implement the IteratorAggregate
	 * interface.  We will return the objects array in this object
	 * @return ArrayObject
	 *
	 */
	public function getIterator() {
		return new ArrayObject($this->getQuestions());
	} // end function getIterator() 
	
	/************************************************************************
							Setter and Getters
	************************************************************************/
	/**
	 * Set the type of list
	 *
	 * This method will make sure that the passed value is one of: angelology, 
	 * hamartiology, anthropology, or inquisition
	 * @param string $string
	 */
	public function setType($string) {
		// Make sure that it's a string
		$string = (string) $string;
		$this->type = $string;
		return true;
	} // end function setType()
	
	/**
	 * Get the type 
	 * @return string
	 */
	public function getType() {
		return $this->type;
	} // end function getType()
	
	/************************************************************************
							Array Accessors 
	************************************************************************/
	/**
	 * Add a question to the questions array
	 * 
	 * This method will check that the data passed to it will create a valid
	 * question, and then, if so, add it to the list of questions.
	 * @param Question $question
	 * @return bool
	 */
	public function addQuestion(Question $question) {
		// Make sure that this question comes from the database, so it's valid
		if ($question->getId() == '') return false;
		
		// Add this question to the list
		$this->questions[] = $question;
		return true;
	} // end function addQuestion()
	
	/**
	 * Remove a question
	 *
	 * If the given question is vlid, it will remove that question from
	 * from the current list
	 * @param Question $question
	 * @return bool
	 */
	public function removeQuestion($question) {
		// Make sure that this question comes from the database, so it's valid
		if ($question->getId() == '') return false;
		// Walk through the questions looking for this one
		foreach ($this->getQuestions() as $key=>$quest) {
			if ($quest->getId() == $question->getId()) {
				unset($this->questions[$key]);
				return true;
			} // end actions if we found the question to remove
		} // end removal of the answer
		return false;
	} // end function removeQuestion()
	
	/**
	 * Compare two questions and see which comes first
	 * @param array $first
	 * @param array $second
	 * @return int
	 */
	private function compareQuestions($first, $second) {
		return strcmp($first->getDifficulty(), $second->getDifficulty());
	} // end function compareQuestions()
	
	/**
	 * Reorder the Questions so that they are in the right order
	 * @return void
	 */
	public function orderQuestions() {
		return usort($this->questions, array('QuestionList', 'compareQuestions'));
	} // end function orderAnswers
	
	/**
	 * Get all of the questions 
	 * @return array
	 */
	public function getQuestions() {
		return $this->questions;
	} // end function getQuestions()
	
	/**
	 * Clear all of the questions in this list
	 * @return true
	 */
	public function clearQuestions() {
		$this->questions = array();
	} // end function clearQuestions()

	/************************************************************************
							Database Interactions 
	************************************************************************/
	/**
	 * Load Questions from the database for a type
	 *
	 * Given a type of set of questions, this method will query the database for
	 * easy, medium, and hard questions of that type.  The questions will then
	 * be created and added to this list.
	 * @param string $type
	 * @return bool
	 */
	public function loadType($type) {
		// First, see if the type given is valid
		$test = $this->setType($type);
		if (!$test) return false;
		
		// Now, create the database query to get the questions for the type
		$sql = "SELECT id,
					   difficulty
				FROM questions
				WHERE class = '" . pg_escape_string($type) . "'";
		// Query the database
		$select = pg_query($GLOBALS['_SCRIPT']['database'], $sql);
		// If it didn't work, give up
		if (!$select) return false;
		
		// Clear all of the questions already in this list
		$this->clearQuestions();
		
		// Create a list of each of the easy, medium, and hard questions
		$count = pg_num_rows($select);
		$easy = array();
		$med = array();
		$hard = array();

		for ($iterate = 0; $iterate < $count; $iterate++) {
			$row = pg_fetch_array($select, $iterate, PGSQL_ASSOC);
			switch ($row['difficulty']) {
				case 0:
					$easy[$row['id']] = $row['id'];
					break;
				case 1: 
					$med[$row['id']] = $row['id'];
					break;
				case 2:
					$hard[$row['id']] = $row['id'];
					break;
			} // end check for the correct type of difficulty
		} // end walk through the results creating the lists of questions
		
		// Choose 5 random questions for each difficulty level
		if (count($easy) >= 5) $easy = array_rand($easy, 5);
		if (count($med) >= 5) $med = array_rand($med, 5);
		if (count($hard) >= 5) $hard = array_rand($hard, 5);
		
		// Walk through each of the questions and add them to the master list
		$success = true;
		foreach ($easy as $key=>$id) {
			$question = new Question;
			$test = $question->loadFromDb($id);
			if ($test) $test = $this->addQuestion($question);
			if (!$test) $success = false;
		} // end walk through the easy array
		foreach ($med as $key=>$id) {
			$question = new Question;
			$test = $question->loadFromDb($id);
			if ($test) $test = $this->addQuestion($question);
			if (!$test) $success = false;
		} // end walk through the medium array
		foreach ($hard as $key=>$id) {
			$question = new Question;
			$test = $question->loadFromDb($id);
			if ($test) $test = $this->addQuestion($question);
			if (!$test) $success = false;
		} // end walk through the hard array
		
		// After all of this, they should be in order, but we'll orderize them
		// once morem, just to be sure
		$this->orderQuestions();
		
		return $success;
		
	} // end function loadType
	
	/************************************************************************
							Interaction Methods 
	************************************************************************/
	/**
	 * Find the next unanswered question
	 *
	 * This method should be used when a new question is needed.
	 * @return Question|false
	 */
	public function nextUnanswered() {
		foreach ($this->getQuestions() as $key=>$question) {
			if ($question->getIsCorrect() === NULL) return $question;
		} // end foreaching through all questions
		return false;
	} // end funciton nextUnanswered()
	
	/**
	 * Find out if all questions have been answered
	 *
	 * This method will return true if all questions have been answered, and 
	 * false if there are some yet to be answered.
	 * @return bool
	 */
	public function getIsComplete() {
		$return = true;
		foreach ($this->getQuestions() as $question) {
			if ($question->getIsCorrect() === NULL) $return = false;
		} // end walk through the questions looking for an unanswered
		return $return;
	} // end function getIsComplete()
	
	/**
	 * Return how many have been answered incorrectly
	 *
	 * This method will return the number of questions which are incorrect 
	 * @return int
	 */
	public function getIncorrect() {
		$return = 0;
		foreach ($this->getQuestions() as $question) {
			if ($question->getIsCorrect() === false) $return++;
		} // end walk through the questions looking for the incorrect ones
		return $return;
	} // end function getIncorrect()
	
	/**
	 * Return the percent in this list that have been answered
	 *
	 * This method will return the percent of questions in this list which have
	 * been answered, either correct or incorrect.
	 * @return float
	 */
	public function percentAnswered() {
		$answered = 0;
		foreach ($this->getQuestions() as $key=>$question) {
			if ($question->getIsCorrect() !== NULL) $answered++;
		} // end walk through the questions
		return ($answered/count($this->getQuestions())) * 100;
	} // end function percentAnswered()
	
} // end creation of class QuestionList